import * as http from 'http';
import { IncomingHttpHeaders } from 'http';
import { HighlightOptions } from 'highlight.run';
import { GraphQLClient } from 'graphql-request';
import * as Dom from 'graphql-request/dist/types.dom';
import { BaseContext, ApolloServerPlugin } from '@apollo/server';
import { BaseContext as BaseContext$1, ApolloServerPlugin as ApolloServerPlugin$1 } from 'apollo-server-plugin-base';

interface NodeOptions extends HighlightOptions {
    /**
     * ID used to associate payloads with a Highlight project.
     */
    projectID: string;
    /**
     * The endpoint string to send opentelemetry data to.
     * @default https://otel.highlight.io:4318
     */
    otlpEndpoint?: string;
}

declare const HIGHLIGHT_REQUEST_HEADER = "x-highlight-request";
interface HighlightInterface {
    init: (options: NodeOptions) => void;
    isInitialized: () => boolean;
    parseHeaders: (headers: IncomingHttpHeaders) => {
        secureSessionId: string;
        requestId: string;
    } | undefined;
    consumeError: (error: Error, secureSessionId?: string, requestId?: string) => void;
    consumeEvent: (secureSessionId?: string) => void;
    recordMetric: (secureSessionId: string, name: string, value: number, requestId?: string, tags?: {
        name: string;
        value: string;
    }[]) => void;
    flush: () => Promise<void>;
    log: (...data: any[]) => void;
}
declare const H: HighlightInterface;

declare type Maybe<T> = T | null;
declare type InputMaybe<T> = Maybe<T>;
declare type Exact<T extends {
    [key: string]: unknown;
}> = {
    [K in keyof T]: T[K];
};
/** All built-in and custom scalars, mapped to their actual values */
declare type Scalars = {
    ID: string;
    String: string;
    Boolean: boolean;
    Int: number;
    Float: number;
    Any: any;
    Int64: any;
    Timestamp: any;
};
declare type MetricInput = {
    category?: InputMaybe<Scalars['String']>;
    group?: InputMaybe<Scalars['String']>;
    name: Scalars['String'];
    session_secure_id: Scalars['String'];
    tags?: InputMaybe<Array<MetricTag>>;
    timestamp: Scalars['Timestamp'];
    value: Scalars['Float'];
};
declare type MetricTag = {
    name: Scalars['String'];
    value: Scalars['String'];
};
declare type MarkBackendSetupMutationVariables = Exact<{
    project_id: Scalars['String'];
    session_secure_id?: InputMaybe<Scalars['String']>;
}>;
declare type MarkBackendSetupMutation = {
    __typename?: 'Mutation';
    markBackendSetup?: any | null;
};
declare type PushMetricsMutationVariables = Exact<{
    metrics: Array<InputMaybe<MetricInput>> | InputMaybe<MetricInput>;
}>;
declare type PushMetricsMutation = {
    __typename?: 'Mutation';
    pushMetrics: number;
};
declare type SdkFunctionWrapper = <T>(action: (requestHeaders?: Record<string, string>) => Promise<T>, operationName: string, operationType?: string) => Promise<T>;
declare function getSdk(client: GraphQLClient, withWrapper?: SdkFunctionWrapper): {
    MarkBackendSetup(variables: MarkBackendSetupMutationVariables, requestHeaders?: Dom.RequestInit['headers']): Promise<MarkBackendSetupMutation>;
    PushMetrics(variables: PushMetricsMutationVariables, requestHeaders?: Dom.RequestInit['headers']): Promise<PushMetricsMutation>;
};
declare type Sdk = ReturnType<typeof getSdk>;

declare class Highlight {
    readonly FLUSH_TIMEOUT = 10;
    readonly BACKEND_SETUP_TIMEOUT: number;
    _graphqlSdk: Sdk;
    _backendUrl: string;
    _intervalFunction: ReturnType<typeof setInterval>;
    metrics: Array<InputMaybe<MetricInput>>;
    lastBackendSetupEvent: number;
    _projectID: string;
    _debug: boolean;
    private otel;
    private tracer;
    constructor(options: NodeOptions);
    _log(...data: any[]): void;
    recordMetric(secureSessionId: string, name: string, value: number, requestId?: string, tags?: {
        name: string;
        value: string;
    }[]): void;
    log(date: Date, msg: string, level: string, stack: object, secureSessionId?: string, requestId?: string): void;
    consumeCustomError(error: Error, secureSessionId: string | undefined, requestId: string | undefined): void;
    consumeCustomEvent(secureSessionId?: string): void;
    flushMetrics(): Promise<void>;
    flush(): Promise<void>;
}

declare const ApolloServerHighlightPlugin: <T extends BaseContext>(options: NodeOptions) => ApolloServerPlugin<T>;

declare const ApolloServerV3HighlightPlugin: <T extends BaseContext$1>(options: NodeOptions) => ApolloServerPlugin$1<T>;

/** JSDoc */
interface MiddlewareError extends Error {
    status?: number | string;
    statusCode?: number | string;
    status_code?: number | string;
    output?: {
        statusCode?: number | string;
    };
}
/**
 * Express compatible error handler.
 * Exposed as `Handlers.errorHandler`
 */
declare function errorHandler(options: NodeOptions): (error: MiddlewareError, req: http.IncomingMessage, res: http.ServerResponse, next: (error: MiddlewareError) => void) => void;
/**
 * A TRPC compatible error handler for logging errors to Highlight.
 */
declare function trpcOnError({ error, req, }: {
    error: Error;
    req: {
        headers?: http.IncomingHttpHeaders;
    };
}, options: NodeOptions): Promise<void>;
/**
 * A wrapper for logging errors to Highlight for Firebase HTTP functions
 */
declare type FirebaseHttpFunctionHandler = (req: any, resp: any) => void | Promise<void>;
declare function firebaseHttpFunctionHandler(origHandler: FirebaseHttpFunctionHandler, options: NodeOptions): FirebaseHttpFunctionHandler;
/**
 * A wrapper for logging errors to Highlight for Firebase callable functions
 */
declare type FirebaseCallableFunctionHandler = (data: any, context: any) => any;
declare function firebaseCallableFunctionHandler(origHandler: FirebaseCallableFunctionHandler, options: NodeOptions): FirebaseCallableFunctionHandler;

declare const handlers_errorHandler: typeof errorHandler;
declare const handlers_trpcOnError: typeof trpcOnError;
declare const handlers_firebaseHttpFunctionHandler: typeof firebaseHttpFunctionHandler;
declare const handlers_firebaseCallableFunctionHandler: typeof firebaseCallableFunctionHandler;
declare namespace handlers {
  export {
    handlers_errorHandler as errorHandler,
    handlers_trpcOnError as trpcOnError,
    handlers_firebaseHttpFunctionHandler as firebaseHttpFunctionHandler,
    handlers_firebaseCallableFunctionHandler as firebaseCallableFunctionHandler,
  };
}

export { ApolloServerHighlightPlugin, ApolloServerV3HighlightPlugin, H, HIGHLIGHT_REQUEST_HEADER, handlers as Handlers, Highlight, NodeOptions };
