var __defProp = Object.defineProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// src/log.ts
var start = Date.now();
function log(from, ...data) {
  const prefix = `[Highlight Node.js ${(Date.now() - start) / 1e3}s] - {${from}}`;
  console.log.apply(console, [prefix, ...data]);
}

// src/sdk.ts
var HIGHLIGHT_REQUEST_HEADER = "x-highlight-request";
var _debug = false;
var highlight_obj;
var H = {
  init: (options) => {
    _debug = !!options.debug;
    try {
      highlight_obj = new Highlight(options);
    } catch (e) {
      console.warn("highlight-node init error: ", e);
    }
  },
  isInitialized: () => !!highlight_obj,
  consumeError: (error, secureSessionId, requestId) => {
    try {
      highlight_obj.consumeCustomError(error, secureSessionId, requestId);
    } catch (e) {
      console.warn("highlight-node consumeError error: ", e);
    }
  },
  consumeEvent: (secureSessionId) => {
    try {
      highlight_obj.consumeCustomEvent(secureSessionId);
    } catch (e) {
      console.warn("highlight-node consumeEvent error: ", e);
    }
  },
  recordMetric: (secureSessionId, name, value, requestId, tags) => {
    try {
      highlight_obj.recordMetric(
        secureSessionId,
        name,
        value,
        requestId,
        tags
      );
    } catch (e) {
      console.warn("highlight-node recordMetric error: ", e);
    }
  },
  flush: async () => {
    try {
      await highlight_obj.flush();
    } catch (e) {
      console.warn("highlight-node flush error: ", e);
    }
  },
  parseHeaders: (headers) => {
    try {
      if (headers && headers[HIGHLIGHT_REQUEST_HEADER]) {
        const [secureSessionId, requestId] = `${headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
        return { secureSessionId, requestId };
      } else {
        H.log(
          `request headers do not contain ${HIGHLIGHT_REQUEST_HEADER}`
        );
      }
    } catch (e) {
      console.warn("highlight-node parseHeaders error: ", e);
    }
    return void 0;
  },
  log: (...data) => {
    if (_debug) {
      log("H", ...data);
    }
  }
};

// src/graph/generated/operations.ts
import gql from "graphql-tag";
var MarkBackendSetupDocument = gql`
	mutation MarkBackendSetup(
		$project_id: String!
		$session_secure_id: String
	) {
		markBackendSetup(
			project_id: $project_id
			session_secure_id: $session_secure_id
		)
	}
`;
var PushMetricsDocument = gql`
	mutation PushMetrics($metrics: [MetricInput]!) {
		pushMetrics(metrics: $metrics)
	}
`;
var defaultWrapper = (action, _operationName, _operationType) => action();
function getSdk(client, withWrapper = defaultWrapper) {
  return {
    MarkBackendSetup(variables, requestHeaders) {
      return withWrapper(
        (wrappedRequestHeaders) => client.request(
          MarkBackendSetupDocument,
          variables,
          { ...requestHeaders, ...wrappedRequestHeaders }
        ),
        "MarkBackendSetup",
        "mutation"
      );
    },
    PushMetrics(variables, requestHeaders) {
      return withWrapper(
        (wrappedRequestHeaders) => client.request(
          PushMetricsDocument,
          variables,
          { ...requestHeaders, ...wrappedRequestHeaders }
        ),
        "PushMetrics",
        "mutation"
      );
    }
  };
}

// src/client.ts
import { GraphQLClient } from "graphql-request";
import * as opentelemetry from "@opentelemetry/sdk-node";
import { getNodeAutoInstrumentations } from "@opentelemetry/auto-instrumentations-node";
import { OTLPTraceExporter } from "@opentelemetry/exporter-trace-otlp-http";
import { trace } from "@opentelemetry/api";

// src/hooks.ts
var consoleHooked = false;
function hookConsole(methodsToRecord, cb) {
  if (consoleHooked)
    return;
  consoleHooked = true;
  const levels = {
    debug: "debug",
    info: "info",
    log: "info",
    count: "info",
    dir: "info",
    warn: "warn",
    assert: "warn",
    error: "error",
    trace: "trace"
  };
  for (const [level, highlightLevel] of Object.entries(levels)) {
    if ((methodsToRecord == null ? void 0 : methodsToRecord.length) && methodsToRecord.indexOf(level) === -1) {
      continue;
    }
    const origWrite = console[level];
    console[level] = function(...data) {
      const date = new Date();
      try {
        return origWrite(...data);
      } finally {
        const o = { stack: {} };
        const saveTraceLimit = Error.stackTraceLimit;
        Error.stackTraceLimit = 3;
        Error.captureStackTrace(o);
        Error.stackTraceLimit = saveTraceLimit;
        cb({
          date,
          level: highlightLevel,
          message: data.join("\n"),
          stack: o.stack
        });
      }
    };
  }
}

// src/client.ts
var OTLP_HTTP = "https://otel.highlight.io:4318";
var Highlight = class {
  constructor(options) {
    this.FLUSH_TIMEOUT = 10;
    this.BACKEND_SETUP_TIMEOUT = 15 * 60 * 1e3;
    this.metrics = [];
    this.lastBackendSetupEvent = 0;
    this._debug = !!options.debug;
    this._projectID = options.projectID;
    this._backendUrl = options.backendUrl || "https://pub.highlight.run";
    if (!options.disableConsoleRecording) {
      hookConsole(options.consoleMethodsToRecord, (c) => {
        this.log(c.date, JSON.stringify(c.message), c.level, c.stack);
      });
    }
    const client = new GraphQLClient(this._backendUrl, {
      headers: {}
    });
    this._graphqlSdk = getSdk(client);
    this.tracer = trace.getTracer("highlight-node");
    const exporter = new OTLPTraceExporter({
      url: `${options.otlpEndpoint ?? OTLP_HTTP}/v1/traces`
    });
    this.otel = new opentelemetry.NodeSDK({
      traceExporter: exporter,
      instrumentations: [getNodeAutoInstrumentations()]
    });
    this.otel.start();
    this._intervalFunction = setInterval(
      () => this.flush(),
      this.FLUSH_TIMEOUT * 1e3
    );
    this._graphqlSdk.MarkBackendSetup({
      project_id: this._projectID
    }).then(() => {
      this.lastBackendSetupEvent = Date.now();
    }).catch((e) => {
      console.warn("highlight-node error: ", e);
    });
    this._log(`Initialized SDK for project ${this._projectID}`);
  }
  _log(...data) {
    if (this._debug) {
      log("client", ...data);
    }
  }
  recordMetric(secureSessionId, name, value, requestId, tags) {
    this.metrics.push({
      session_secure_id: secureSessionId,
      group: requestId,
      name,
      value,
      category: "BACKEND",
      timestamp: new Date().toISOString(),
      tags
    });
  }
  log(date, msg, level, stack, secureSessionId, requestId) {
    if (!this.tracer)
      return;
    const span = this.tracer.startSpan("highlight-ctx");
    span.addEvent(
      "log",
      {
        ["highlight.project_id"]: this._projectID,
        ["code.stack"]: JSON.stringify(stack),
        ["log.severity"]: level,
        ["log.message"]: msg,
        ...secureSessionId ? {
          ["highlight.session_id"]: secureSessionId
        } : {},
        ...requestId ? {
          ["highlight.trace_id"]: requestId
        } : {}
      },
      date
    );
    span.end();
  }
  consumeCustomError(error, secureSessionId, requestId) {
    let span = trace.getActiveSpan();
    let spanCreated = false;
    if (!span) {
      span = this.tracer.startSpan("highlight-ctx");
      spanCreated = true;
    }
    span.recordException(error);
    span.setAttributes({
      ["highlight.project_id"]: this._projectID,
      ["highlight.session_id"]: secureSessionId,
      ["highlight.trace_id"]: requestId
    });
    if (spanCreated) {
      span.end();
    }
  }
  consumeCustomEvent(secureSessionId) {
    const sendBackendSetup = Date.now() - this.lastBackendSetupEvent > this.BACKEND_SETUP_TIMEOUT;
    if (sendBackendSetup) {
      this._graphqlSdk.MarkBackendSetup({
        project_id: this._projectID,
        session_secure_id: secureSessionId
      }).then(() => {
        this.lastBackendSetupEvent = Date.now();
      }).catch((e) => {
        console.warn("highlight-node error: ", e);
      });
    }
  }
  async flushMetrics() {
    if (this.metrics.length === 0) {
      return;
    }
    const variables = {
      metrics: this.metrics
    };
    this.metrics = [];
    try {
      await this._graphqlSdk.PushMetrics(variables);
    } catch (e) {
      console.warn("highlight-node pushMetrics error: ", e);
    }
  }
  async flush() {
    await Promise.all([this.flushMetrics()]);
  }
};

// src/apollo.ts
var ApolloServerHighlightPlugin = function(options) {
  return {
    async requestDidStart(req) {
      var _a, _b, _c;
      let secureSessionId;
      let requestId;
      if ((_b = (_a = req.request.http) == null ? void 0 : _a.headers) == null ? void 0 : _b.get(HIGHLIGHT_REQUEST_HEADER)) {
        ;
        [secureSessionId, requestId] = `${(_c = req.request.http.headers) == null ? void 0 : _c.get(
          HIGHLIGHT_REQUEST_HEADER
        )}`.split("/");
      }
      H.log("processError", "extracted from headers", {
        secureSessionId,
        requestId
      });
      if (!H.isInitialized()) {
        H.init(options);
        H.log("initialized H in apollo server");
      }
      return {
        async didEncounterErrors(requestContext) {
          H.consumeEvent(secureSessionId);
          for (const error of requestContext.errors) {
            H.consumeError(error, secureSessionId, requestId);
            H.log("consumed apollo request error", error);
          }
        }
      };
    }
  };
};

// src/apolloV3.ts
var ApolloServerV3HighlightPlugin = function(options) {
  return {
    async requestDidStart(req) {
      var _a, _b, _c;
      let secureSessionId;
      let requestId;
      if ((_b = (_a = req.request.http) == null ? void 0 : _a.headers) == null ? void 0 : _b.get(HIGHLIGHT_REQUEST_HEADER)) {
        ;
        [secureSessionId, requestId] = `${(_c = req.request.http.headers) == null ? void 0 : _c.get(
          HIGHLIGHT_REQUEST_HEADER
        )}`.split("/");
      }
      H.log("processError", "extracted from headers", {
        secureSessionId,
        requestId
      });
      if (!H.isInitialized()) {
        H.init(options);
        H.log("initialized H in apollo server");
      }
      return {
        async didEncounterErrors(requestContext) {
          H.consumeEvent(secureSessionId);
          for (const error of requestContext.errors) {
            H.consumeError(error, secureSessionId, requestId);
            H.log("consumed apollo request error", error);
          }
        }
      };
    }
  };
};

// src/handlers.ts
var handlers_exports = {};
__export(handlers_exports, {
  errorHandler: () => errorHandler,
  firebaseCallableFunctionHandler: () => firebaseCallableFunctionHandler,
  firebaseHttpFunctionHandler: () => firebaseHttpFunctionHandler,
  trpcOnError: () => trpcOnError
});
function processErrorImpl(options, req, error) {
  let secureSessionId;
  let requestId;
  if (req.headers && req.headers[HIGHLIGHT_REQUEST_HEADER]) {
    ;
    [secureSessionId, requestId] = `${req.headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
  }
  H.log("processError", "extracted from headers", {
    secureSessionId,
    requestId
  });
  if (!H.isInitialized()) {
    H.init(options);
    H.log("initialized H");
  }
  H.consumeEvent(secureSessionId);
  if (error instanceof Error) {
    H.consumeError(error, secureSessionId, requestId);
    H.log("consumed error", error);
  }
}
function errorHandler(options) {
  H.log("setting up error handler");
  return async (error, req, res, next) => {
    H.log("handling request");
    try {
      processErrorImpl(options, req, error);
    } finally {
      next(error);
    }
  };
}
async function trpcOnError({
  error,
  req
}, options) {
  try {
    if (!H.isInitialized()) {
      H.init(options);
    }
    processErrorImpl(options, req, error);
    await H.flush();
  } catch (e) {
    console.warn("highlight-node trpcOnError error:", e);
  }
}
function firebaseHttpFunctionHandler(origHandler, options) {
  return async (req, res) => {
    try {
      return await origHandler(req, res);
    } catch (e) {
      try {
        if (e instanceof Error) {
          if (!H.isInitialized()) {
            H.init(options);
          }
          processErrorImpl(options, req, e);
          await H.flush();
        }
      } catch (e2) {
        console.warn(
          "highlight-node firebaseHttpFunctionHandler error:",
          e2
        );
      }
      throw e;
    }
  };
}
function firebaseCallableFunctionHandler(origHandler, options) {
  return async (data, context) => {
    try {
      return await origHandler(data, context);
    } catch (e) {
      try {
        if (e instanceof Error) {
          if (!H.isInitialized()) {
            H.init(options);
          }
          processErrorImpl(options, context.rawRequest, e);
          await H.flush();
        }
      } catch (e2) {
        console.warn(
          "highlight-node firebaseCallableFunctionHandler error:",
          e2
        );
      }
      throw e;
    }
  };
}
export {
  ApolloServerHighlightPlugin,
  ApolloServerV3HighlightPlugin,
  H,
  HIGHLIGHT_REQUEST_HEADER,
  handlers_exports as Handlers,
  Highlight
};
